'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fieldList = require('./field-list');

var _fieldList2 = _interopRequireDefault(_fieldList);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getInternal = require('internal-data')(); // eslint-disable-line no-use-before-define

/**
 * Private attributes of object.
 * @typedef   {Object}          Validator~Internal
 * @private
 * @property  {Fields}  whiteList             - Rule for allowed fields and relations.
 * @property  {Fields}  blackList             - Rule for disallowed fields and relations.
 */


/**
 * Class which validates database fields using white list and black list.
 */
class AllowedFields {
  /**
   * Creates object.
   * @param     {Fields}  [whiteList]     - List of allowed identifiers (entities and fields) to be used in query.
   * @param     {Fields}  [blackList]     - List of identifiers which are prohibited to use in query.
   * @example
   * const employees = new AllowedFields(
   *   whiteList: { '': 'color', member: '*', manager: ['name', 'surname'] },
   *   blackList: { member: ['salary', 'id'] },
   * );
   * employees.isAllowed('member.name');    // true
   * employees.isAllowed('color');          // true
   * employees.isAllowed('manager.name');   // true
   * employees.isAllowed('member.salary');  // false
   * employees.isAllowed('manager.salary'); // false
   */
  constructor({ whiteList, blackList }) {
    const internal = getInternal(this);

    internal.whiteList = whiteList ? new _fieldList2.default(whiteList) : undefined;
    internal.blackList = blackList ? new _fieldList2.default(blackList) : undefined;
  }

  /**
   * Returns whether given field/relation combination is an allowed field according to given rules.
   * Field name can be provided in single parameter or two parameters: i.e ('name', 'member')  or ('member.name').
   * @param     {string}          field           - Field name to test. i.e `'name'`. Also it may contain field name such as 'member.name'
   * @param     {string}          [relation='']   - Relation name which field belongs to.
   * @returns   {boolean}                         - Whether field is valid.
   * @example
   * allowedFields.isAllowed('member.name');
   * allowedFields.isAllowed('name', 'member');
   */
  isAllowed(field, relation = '') {
    const internal = getInternal(this);
    const { whiteList, blackList } = internal;

    if (field.match(/\./)) {
      [relation, field] = field.split('.');
    }

    // * return false. Also relation.* returns false if any field of the relation is in blacklist.
    const starError = field === '*' && (!relation || blackList && blackList.hasRelation(relation));
    const whiteListError = whiteList && !whiteList.has(field, relation);
    const blackListError = blackList && blackList.has(field, relation);

    return !(starError || whiteListError || blackListError);
  }
}

exports.default = AllowedFields;
module.exports = exports['default'];
//# sourceMappingURL=index.js.map